<!-- wp:paragraph -->
<p><em>{{root.description}}</em></p>
<!-- /wp:paragraph -->

<!-- wp:paragraph -->
<p><code>{% if root.name -%}{{root.name}}{%-else-%}{{root.identifier}}{%-endif%}</code></p>
<!-- /wp:paragraph -->

<!-- wp:paragraph -->
<p><a href="#resources">Resources</a> | <a href="#packages">Packages</a> | <a href="#documentation">Documentation</a>
{%- if contacts -%} | <a href="#contacts">Contacts</a>{%- endif -%}
{%- if references -%} | <a href="#references">References</a>{%- endif -%}
{%- if schema -%} | <a href="#data_dictionary">Data Dictionary</a>{%- endif -%}</p>
<!-- /wp:paragraph -->